import base64

from django.contrib.auth.models import User
from django.test import TestCase
from rest_framework.test import APIClient


class BaseTestCase(TestCase):
    """A base test case."""

    def setUp(self):
        self.client = APIClient()
        self.user = \
            User.objects.create(username="my_user", password="my_passwd")

    def tearDown(self):
        pass

    def basics_auth_get(self, url, username, password,
                        data=None, content_type='application/json'):
        valid_credentials = base64.b64encode(
            bytes(username + ":" + password, 'ascii')).decode('ascii')
        return self.client.get(
            url,
            headers={
                'Authorization': 'Basic ' + valid_credentials
            },
            content_type=content_type,
            data=data
        )

    def basics_auth_post(self, url, username, password, data, format='json'):
        valid_credentials = base64.b64encode(
            bytes(username + ":" + password, 'ascii')).decode('ascii')
        # see http://www.django-rest-framework.org/api-guide/testing/
        return self.client.post(
            url,
            headers={
                'Authorization': 'Basic ' + valid_credentials
            },
            format=format,
            data=data
        )

    def token_auth_get(self, url, token,
                       data=None, content_type='application/json'):
        return self.client.get(
            url,
            headers={
                'Authorization': 'Token ' + token
            },
            content_type=content_type,
            data=data
        )

    def token_auth_post(self, url, token, data, format='json'):
        return self.client.post(
            url,
            headers={
                'Authorization': 'Token ' + token
            },
            format=format,
            data=data
        )



"""project URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from drf-microservice import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.conf import settings
from django.conf.urls.static import static
from django.urls import path, re_path, include
from django.contrib import admin
from rest_framework import permissions
from rest_framework.authtoken import views as rest_framework_views
from rest_framework.documentation import include_docs_urls
from drf_yasg.views import get_schema_view
from drf_yasg import openapi


from my_api.rest.views import status_api, Icinga2API, FileAPI
if settings.DEBUG_URL:
    from my_api.rest.views import \
        api_handler400, api_handler403, api_handler404, api_handler500


schema_view = get_schema_view(
    openapi.Info(
        title="API",
        default_version='v1',
        description="DRF Microservice RESTFUL",
        terms_of_service="/terms/",
        contact=openapi.Contact(email="alain.ivars@gmail.com"),
    ),
    public=True,
    permission_classes=(permissions.AllowAny,),
)

urlpatterns = [
    re_path(
        r'^swagger/openapi(?P<format>\.json|\.yaml)$',
        schema_view.without_ui(cache_timeout=0),
        name='schema-json-or-yaml'
    ),
    re_path(
        r'^swagger/openapi/$',
        schema_view.with_ui('swagger', cache_timeout=0),
        name='schema-openapi-ui'
    ),
    re_path(
        r'^swagger/redoc/$',
        schema_view.with_ui('redoc', cache_timeout=0),
        name='schema-redoc'
    ),
]

urlpatterns += [
    path('admin/', admin.site.urls),

    # Authentication, Authorization, Users, Groups
    re_path(r'^api-auth/', include('dj_rest_auth.urls')),
    re_path(
        r'^api-auth-token/$',
        rest_framework_views.obtain_auth_token,
        name='get_auth_token'
    ),

    # Documentation
    re_path(r'^docs/', include_docs_urls(title='drf-microservice API')),

    # Probe live control
    path('icinga/', status_api, name='icinga'),
    path('icinga2/', Icinga2API.as_view(), name='icinga2'),

    # the API
    path('api/v1/file', FileAPI.as_view(), name='api_file'),

    # Some media files if you need it else remove it
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

if settings.DEBUG_URL:
    # To debug the error pages during development
    urlpatterns += [
        re_path(r'^400/$', api_handler400, name='handler400'),
        re_path(r'^403/$', api_handler403, name='handler403'),
        re_path(r'^404/$', api_handler404, name='handler404'),
        re_path(r'^500/$', api_handler500, name='handler500'),
    ]
    # static/media files
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
    urlpatterns += static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)

handler400 = 'my_api.rest.views.api_handler400'
handler403 = 'my_api.rest.views.api_handler403'
handler404 = 'my_api.rest.views.api_handler404'
handler500 = 'my_api.rest.views.api_handler500'



# Load the production settings that we will overwrite as needed in our user1 settings file.
from my_api.config.production import *

ALLOWED_HOSTS.extend(['127.0.0.1', 'localhost', '0.0.0.0'])
DEBUG_URL = 1



import copy
import json
import uuid

from django.contrib.auth.models import User, Group
from rest_framework import serializers
from rest_framework.authtoken.models import Token


class UserSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = User
        fields = ('url', 'username', 'email', 'groups')
        read_only_fields = ('username', )

    # def create(self, validated_data):
    #     user = User(
    #         email=validated_data['email'],
    #         username=validated_data['username']
    #     )
    #     user.set_password(validated_data['password'])
    #     user.save()
    #     Token.objects.create(user=user)
    #     return user


class GroupSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Group
        fields = ('url', 'name')


# class FileSerialised:
#     """
#     In case we want to store a list in db
#     """
#     class Meta:
#         model = FileSerialised
#         fields = ('filename', 'header', 'file')


class FileToFilesystemSerializer(serializers.BaseSerializer):
    path = None

    def __init__(self, *args, **kwargs):
        assert 'path' in kwargs and kwargs['path'] is not None, \
            '`path` is a required argument.'
        self.path = kwargs.pop('path', copy.deepcopy(self.path))
        super(FileToFilesystemSerializer, self).__init__(*args, **kwargs)

    def create(self, validated_data):
        # data = str(validated_data) #validated_data['file']
        # extension = validated_data['extension']
        filename = str(uuid.uuid1())
        # a way to know if the same file is already stored !!
        # filename = str(hash(validated_data['file']))
        with open(self.path + filename + '.json', 'w') as f:
            json.dump(validated_data, f)
        # In case we want to store a list in db
        # return FileSerialised(filename, extension, data)
        return {'name': filename}

    def update(self, instance, validated_data):
        raise NotImplementedError('`update()` must be implemented.')

    def to_internal_value(self, data):
        if isinstance(data, dict) and 'file' in data:
            return data
        else:
            raise serializers.ValidationError("'file' is a required field.")

    def to_representation(self, obj):
        raise NotImplementedError('`to_representation()` must be implemented.')
        # return obj.data



import json

from django.conf import settings
from django.http import JsonResponse
from rest_framework.decorators import api_view
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status

from docs.version import __version__
from my_api.rest.serializers import FileToFilesystemSerializer


"""
Official drf doc here:
https://www.django-rest-framework.org/api-guide/requests/#authenticators
"""


@api_view(['GET'])
def status_api(request):
    """
    For Icinga2 or Nagios. Function based.
    Return 'status': 'OK' and the current version running.
    """
    if request.method == 'GET':
        return JsonResponse({
            'status': 'OK',
            'version': __version__
        }, status=200)


class Icinga2API(APIView):
    """
    For Icinga2 or Nagios. Class based.
    Return 'status': 'OK' and the current version running.
    """
    def get(self, request):
        return Response({
            'status': 'OK',
            'version': __version__
        }, status=200)


class FileAPI(APIView):
    """
    Example API to push a json image on the server and get it by a key.
    """
    # def delete(self, request):
    #     """
    #     Required that the client is authenticated,
    #     This method delete a file on the disk (NotImplemented)
    #     :param request: the key corresponding to that file.
    #     :return: status
    #     """
    #     raise NotImplemented()

    def get(self, request):
        """
        Required that the client is authenticated,
        This method load a file from the disk to the json send it in the
        response
        :param request: the key corresponding to that file.
        :return: the file in json Base64 format
        """
        # http://www.django-rest-framework.org/api-guide/requests/#query_params
        if 'name' not in request.query_params \
                or not request.query_params['name']:
            return Response({'error': 'file Not found'}, status=422)
        else:
            name = request.query_params['name']

        with open(settings.PATH_TO_STORE_FILE + name + '.json') as f:
            json_file = json.load(f)
        return Response(json_file, status=200)

    def post(self, request):
        """
        Required that the client is authenticated,
        This method load a file from the json and save it on the disk
        :param request: the file in json Base64 format
        :return: the key to get to that file after.
        """
        serializer = FileToFilesystemSerializer(
            path=settings.PATH_TO_STORE_FILE, data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.instance,
                            status=status.HTTP_201_CREATED)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    # def put(self, request):
    #     """
    #     Required that the client is authenticated,
    #     This method load a file from the json and update it on the disk
    #     (NotImplemented)
    #     :param request: the key and the new file in json Base64 format
    #     :return: the key to get to that file after.
    #     """
    #     raise NotImplemented()


def api_handler400(request, *args, **kwargs):
    data = {
        'error': 'That\'s a BAD REQUEST, try again (400)'
    }
    return JsonResponse(data, status=status.HTTP_400_BAD_REQUEST)


def api_handler403(request, *args, **kwargs):
    data = {
        'error': 'This is FORBIDDEN, you certainly forgot to take the right Token (403)'
    }
    return JsonResponse(data, status=status.HTTP_403_FORBIDDEN)


def api_handler404(request, *args, **kwargs):
    data = {
        'error': 'I do NOT FOUND it! sorry ...(404)'
    }
    return JsonResponse(data, status=status.HTTP_404_NOT_FOUND)


def api_handler500(request, *args, **kwargs):
    data = {
        'error': 'INTERNAL SERVER ERROR, Allo Huston we got a problem! (500)'
    }
    return JsonResponse(data, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



import json
import unittest

from django.contrib.auth.models import User
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APIClient

from my_api.rest.tests.base import BaseTestCase


class ViewTestCase(BaseTestCase):
    """
    Test suite for the api views.
    Infos: Lot of help at
    https://www.django-rest-framework.org/api-guide/testing/#apiclient
    """

    def setUp(self):
        """Define the test client and other test variables."""
        # self.path = reverse('rest_login')
        self.username = "my_user"
        self.password = "my_passwd"
        self.user = User.objects.create(
            username=self.username, password=self.password)
        # Initialize client and force it to use authentication
        self.client = APIClient()
        # self.client.force_authenticate(
        #     user=self.user, token=self.user.auth_token)
        # self.url = reverse('user-detail', kwargs={'pk': self.user.pk})
        self.client.credentials(HTTP_AUTHORIZATION=f'Token {self.user.auth_token}')

    def test_access_with_correct_credential(self):
        response = self.basics_auth_get(
            reverse('icinga'),
            username=self.username,
            password=self.password
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_access_with_correct_token(self):
        response = self.token_auth_get(
            reverse('icinga'),
            str(self.user.auth_token)
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_docs(self):
        response = self.client.get('/docs/', content_type='application/json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_api_file_with_credential_good_key(self):
        with open('./my_api/rest/tests/files/file_ok.json') as f:
            file_content = f.read()
            json_file_content = json.loads(file_content)
        response = self.basics_auth_post(
            reverse('api_file'),
            self.username,
            self.password,
            data=json_file_content
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        data = dict(response.data)
        response = self.basics_auth_get(
            reverse('api_file'),
            self.username,
            self.password,
            data=data
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, json_file_content)

    def test_api_file_with_credential_wrong_key(self):
        with open('./my_api/rest/tests/files/file_ko.json') as f:
            file_content = f.read()
            json_file_content = json.loads(file_content)
        response = self.basics_auth_post(
            reverse('api_file'),
            self.username,
            self.password,
            data=json_file_content
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data[0], "'file' is a required field.")

    def test_api_file_with_token_good_key(self):
        with open('./my_api/rest/tests/files/file_ok.json') as f:
            file_content = f.read()
            json_file_content = json.loads(file_content)
        response = self.token_auth_post(
            reverse('api_file'),
            str(self.user.auth_token),
            data=json_file_content
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        data = response.data
        response = self.token_auth_get(
            reverse('api_file'),
            str(self.user.auth_token),
            data=data
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, json_file_content)

    def test_api_file_with_token_good_key_bad_param(self):
        with open('./my_api/rest/tests/files/file_ok.json') as f:
            file_content = f.read()
            json_file_content = json.loads(file_content)
        response = self.token_auth_post(
            reverse('api_file'),
            str(self.user.auth_token),
            data=json_file_content
        )
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        # data = response.data
        response = self.token_auth_get(
            reverse('api_file'),
            str(self.user.auth_token),
            data={}
        )
        self.assertEqual(response.status_code,
                         status.HTTP_422_UNPROCESSABLE_ENTITY)

    def test_api_file_with_token_wrong_key(self):
        with open('./my_api/rest/tests/files/file_ko.json') as f:
            file_content = f.read()
            json_file_content = json.loads(file_content)
        response = self.token_auth_post(
            reverse('api_file'),
            str(self.user.auth_token),
            data=json_file_content
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(response.data[0], "'file' is a required field.")

    def test_icinga(self):
        response = self.basics_auth_get(
            reverse('icinga'),
            self.username,
            self.password
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_icinga2(self):
        response = self.basics_auth_get(
            reverse('icinga2'),
            self.username,
            self.password
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)


if __name__ == '__main__':
    unittest.main()



"""
Django settings for project 'my_api.

Generated by 'django-admin startproject' using Django 2.1.1.

For more information on this file, see
https://docs.djangoproject.com/en/2.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.1/ref/settings/

List of settings that the project as to have to be ready for production:

- General settings:
    - SECRET_KEY: the Django secret key
    - ENABLE_DEBUG: False by default, set to 1 to enable it.
    - ALLOWED_HOSTS: empty by default, use comma-separated list.
    - LOGGING_CONFIG: default None.
    - TIME_ZONE: default 'UTC', list: http://pytz.sourceforge.net/#what-is-utc
    - PATH_TO_STORE_FILE: default BASE_DIR + '/files/'
- DB-related settings:
    - DB_HOST: default 'localhost'
    - DB_USER: default 'user'
    - DB_NAME: default 'project_db'
    - DB_PASS: default 'passwd' in debug mode ssm 'db_password' in production.
    - DB_PORT: default '5432'
- Email settings:
    - alainivars@gmail.com
- Sentry settings:
    - SENTRY_DSN: default is None, set to a valid DSN to enable sentry.
    - SENTRY_RELEASE: default 'local', set to 'stage', 'live', etc.
- AWS settings:
    - TODO
- Silk profiler:
    - ENABLE_SILK: False by default, set to 1 to enable it.

"""

import os
import sys

# import boto3 as boto3
#
# # for the secrets
# aws_client = boto3.client('ssm', region_name='eu-west-1')
# response = aws_client.get_parameters(
#     Names=[
#         'db_password',
#         'email_password',
#         'django_secret_key',
#         'stats_api_key',
#     ],
#     WithDecryption=True
# )
# secrets = {p.get('Name'): p.get('Value') for p in response.get('Parameters')}
secrets = {}

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

PATH_TO_STORE_FILE = os.environ.get(
    'PATH_TO_STORE_FILE', default=BASE_DIR + '/files/')
os.makedirs(PATH_TO_STORE_FILE, exist_ok=True)

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get('SECRET_KEY')
# or with a file
# with open('/etc/secret_key.txt') as f:
#     SECRET_KEY = f.read().strip()

# Are in test mode?
TESTING = 'test' or '-k e2e' in sys.argv

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = bool(int(os.getenv('ENABLE_DEBUG', 0)))
DEBUG_URL = 0
# Allowed hosts
ALLOWED_HOSTS = os.getenv('ALLOWED_HOSTS', '').split(',')
if DEBUG:
    ALLOWED_HOSTS.extend(['127.0.0.1', 'localhost', '0.0.0.0'])

# Email
EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'

ADMINS = (
    ('Author', 'alainivars@gmail.com'),
)

REST_FRAMEWORK = {
    'EXCEPTION_HANDLER': 'rest_framework.views.exception_handler',
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
    'DEFAULT_SCHEMA_CLASS': 'rest_framework.schemas.coreapi.AutoSchema',
    'PAGE_SIZE': int(os.getenv('DJANGO_PAGINATION_LIMIT', 10)),
    'DATETIME_FORMAT': '%Y-%m-%dT%H:%M:%S%z',

    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.BasicAuthentication',  # require for /docs
        'rest_framework.authentication.TokenAuthentication',
    ),
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.IsAuthenticated',
    ),
    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.JSONRenderer',
        'rest_framework.renderers.BrowsableAPIRenderer',
    ),
    'DEFAULT_PARSER_CLASSES': (
        'rest_framework.parsers.JSONParser',
        'rest_framework.parsers.FormParser',
        'rest_framework.parsers.MultiPartParser'
    ),
}

# Application definition
INSTALLED_APPS = [
    # Core authentication framework and its default models.
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'rest_framework',
    'rest_framework.authtoken',
    'dj_rest_auth',
    'drf_yasg',

    'my_api.rest',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    # Manages sessions across requests
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    # Associates users with requests using sessions.
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'my_api.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'my_api.wsgi.application'

if TESTING:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(BASE_DIR, 'tests.sqlite3'),
        }
    }
else:
    if DEBUG:
        if os.getenv('DOCKER_CONTAINER'):
            DB_HOST = 'db'
        else:
            DB_HOST = os.getenv('DB_HOST', '127.0.0.1')
        DB_USER = os.getenv('DB_USER', 'user')
        DB_PASS = os.getenv('DB_PASS', 'passwd')
        DB_NAME = os.getenv('DB_NAME', 'project_db')
        DB_PORT = os.getenv('DB_PORT', '5432')
        # todo: change to postgres later, and refactor this part
        DATABASES = {
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
            },
        }
    else:
        if os.getenv('DOCKER_CONTAINER'):
            DB_HOST = 'db'
        else:
            DB_HOST = os.getenv('DB_HOST', '127.0.0.1')
        DB_USER = os.getenv('DB_USER', 'user')
        DB_PASS = secrets.get('db_password', 'passwd')
        DB_NAME = os.getenv('DB_NAME', 'project_db')
        DB_PORT = os.getenv('DB_PORT', '5432')
        DATABASES = {
            'default': {
                'ENGINE': 'django.db.backends.postgresql_psycopg2',
                'HOST': DB_HOST,
                'PORT': DB_PORT,
                'NAME': DB_NAME,
                'USER': DB_USER,
                'PASSWORD': DB_PASS,
            }
        }

# Password validation
# https://docs.djangoproject.com/en/2.1/ref/settings/#auth-password-validators
AUTH_PASSWORD_VALIDATORS = [
    {   # todo; here for the fanatic pep8 line len max 79, tell me how to solve it???
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.1/topics/i18n/
LANGUAGE_CODE = 'en-us'

# list: http://pytz.sourceforge.net/#what-is-utc
TIME_ZONE = os.environ.get('TIME_ZONE', default='UTC')
USE_I18N = True
USE_L10N = True
USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.1/howto/static-files/
STATIC_ROOT = os.path.join(os.path.dirname(BASE_DIR), 'static')
STATIC_URL = '/static/'

# Some media files if you need it else remove it
MEDIA_ROOT = os.path.join(os.path.dirname(BASE_DIR), 'media')
MEDIA_URL = '/media/'

# # Sentry
# SENTRY_DSN = os.getenv('SENTRY_DSN', None)
# if SENTRY_DSN is not None:
#     import raven
#     SENTRY_RELEASE = os.getenv('SENTRY_RELEASE', 'local')
#
#     INSTALLED_APPS.append('raven.contrib.django.raven_compat')
#     RAVEN_CONFIG = {
#         'dsn': SENTRY_DSN,
#         'release': SENTRY_RELEASE,
#     }

# Once you’ve set up HTTPS, enable the following settings.
# Set this to True to avoid transmitting the CSRF cookie over HTTP accidentally.
# usefull for BasicAuthentication and SessionAuthentication
# CSRF_COOKIE_SECURE = True
# SESSION_COOKIE_SECURE = True

# Logging
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'django.server': {
            '()': 'django.utils.log.ServerFormatter',
            'format': '[%(server_time)s] %(message)s',
        },
        'verbose': {
            'format':
                # todo; here for the fanatic pep8 line len max 79, tell me how to solve it???
                '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'filters': {
        'require_debug_true': {
            '()': 'django.utils.log.RequireDebugTrue',
        },
    },
    'handlers': {
        'django.server': {
            'level': 'INFO',
            'class': 'logging.StreamHandler',
            'formatter': 'django.server',
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django': {
            'handlers': ['console'],
            'propagate': True,
        },
        'django.server': {
            'handlers': ['django.server'],
            'level': 'INFO',
            'propagate': False,
        },
        'django.request': {
            'handlers': ['mail_admins', 'console'],
            'level': 'ERROR',
            'propagate': False,
        },
        'django.db.backends': {
            'handlers': ['console'],
            'level': 'INFO'
        },
    }
}



#!/usr/bin/env python

import os
from setuptools import setup, find_packages
from docs.version import __version__

here = os.path.dirname(os.path.abspath(__file__))
with open(os.path.join(here, 'README.rst')) as f:
    long_description = f.read().strip()

setup(
    name='drf-microservice',
    version=__version__,
    author='Alain IVARS',
    author_email='alainivars@gmail.com',
    url='http://github.com/alainivars/drf-microservice',
    license='Apache License 2.0',
    description='''
    A REST API endpoints with Authentication and Registration.
    Read the README.rst for more information.
    ''',
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type='text/x-rst',
    keywords='''
    django rest auth registration rest-framework django-registration api docker cookiecuter tox pytest
    ''',
    zip_safe=False,
    include_package_data=True,
    python_requires=">=2.7, !=3.6.*, !=3.7.*, !=3.8.*",
    # https://pypi.org/classifiers/
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP'
    ],
)



import unittest

from rest_framework import status
from rest_framework.reverse import reverse

from my_api.rest.tests.base import BaseTestCase


class BasicTestCase(BaseTestCase):

    def test_api_handler400(self):
        response = self.client.get(reverse('handler400'))
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_api_handler403(self):
        response = self.client.get(reverse('handler403'))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_api_handler404(self):
        response = self.client.get(reverse('handler404'))
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_api_handler500(self):
        response = self.client.get(reverse('handler500'))
        self.assertEqual(response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR)

    def test_authorization_is_enforced(self):
        response = self.client.get(reverse('icinga'))
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_access_with_false_credential(self):
        response = self.basics_auth_get(
            reverse('icinga'),
            username="admin",
            password="admin"
        )
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_access_with_false_token(self):
        false_token = "c2dffbda4f73937afb13ce8dc281759a8b"
        response = self.token_auth_get(
            reverse('icinga'),
            false_token
        )
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)


if __name__ == '__main__':
    unittest.main()
